--[[
	LFG MatchMaker - Addon for World of Warcraft.
	Version: 1.0.9
	URL: https://github.com/AvilanHauxen/LFG_MatchMaker
	Copyright (C) 2019-2020 L.I.R.

	This file is part of 'LFG MatchMaker' addon for World of Warcraft.

    'LFG MatchMaker' is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    'LFG MatchMaker' is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with 'LFG MatchMaker'. If not, see <https://www.gnu.org/licenses/>.
]]--


------------------------------------------------------------------------------------------------------------------------
-- LFM TAB
------------------------------------------------------------------------------------------------------------------------
--local vanillaDungeonsList, vanillaRaidList, tbcDungeonList, tbcRaidList, pvpList = {}, {}, {}, {}, {}
function LFGMM_LfmTab_Initialize()
	LFGMM_LfmTab_SearchActiveText.StringAnimation = "";

	LFGMM_Utility_InitializeDropDown(LFGMM_LfmTab_CategoryDropDown, 150, LFGMM_LfmTab_CategoryDropDown_OnInitialize);
	LFGMM_Utility_InitializeDropDown(LFGMM_LfmTab_DungeonsDropDown, 150, LFGMM_LfmTab_DungeonsDropDown_OnInitialize);
	LFGMM_Utility_InitializeDropDown(LFGMM_LfmTab_ModeDropDown, 150, LFGMM_LfmTab_ModeDropDown_OnInitialize);

	LFGMM_LfmTab_StartStopSearchButton:SetScript("OnClick", LFGMM_LfmTab_StartStopSearchButton_OnClick);

	LFGMM_Utility_InitializeCheckbox(LFGMM_LfmTab_MatchLfgCheckBox, "LFG", "Get notifications on LFG messages", true, LFGMM_LfmTab_MatchLfgCheckBox_OnClick);
	LFGMM_Utility_InitializeCheckbox(LFGMM_LfmTab_MatchLfmCheckBox,	"LFM", "Get notifications on LFM messages", LFGMM_DB.SEARCH.LFM.MatchLfm, LFGMM_LfmTab_MatchLfmCheckBox_OnClick);
	LFGMM_Utility_InitializeCheckbox(LFGMM_LfmTab_MatchUnknownCheckBox, "Unknown", "Get notifications when dungeon matches, but LFG/LFM cannot be determined", LFGMM_DB.SEARCH.LFM.MatchUnknown, LFGMM_LfmTab_MatchUnknownCheckBox_OnClick);
	LFGMM_Utility_InitializeCheckbox(LFGMM_LfmTab_AutoStopCheckBox, "Stop search when group is full", "Automatically stop search when the group is full", LFGMM_DB.SEARCH.LFM.AutoStop, LFGMM_LfmTab_AutoStopCheckBox_OnClick);
	LFGMM_Utility_InitializeCheckbox(LFGMM_LfmTab_IgnoreBoostsCheckBox, "Ignore Boosts", "Ignore boost requests", LFGMM_DB.SEARCH.LFM.IgnoreBoosts, LFGMM_LfmTab_IgnoreBoostsCheckBox_OnClick);
	LFGMM_Utility_InitializeCheckbox(LFGMM_LfmTab_EnableBroadcastCheckBox, "Broadcast in LFG channel", "Periodically send LFM messages to LookingForGroup channel", LFGMM_DB.SEARCH.LFM.Broadcast, LFGMM_LfmTab_EnableBroadcastCheckBox_OnClick);

	LFGMM_LfmTab_BroadcastMessageTemplateInputBox:SetScript("OnTextChanged", LFGMM_LfmTab_UpdateBroadcastMessage);
	LFGMM_LfmTab_BroadcastMessageTemplateInputBox:SetText(LFGMM_DB.SEARCH.LFM.BroadcastMessageTemplate);

	LFGMM_Utility_InitializeHiddenSlider(LFGMM_LfmTab_BroadcastMessagePreviewSlider, LFGMM_LfmTab_BroadcastMessagePreview_Refresh);
	LFGMM_LfmTab_BroadcastMessagePreviewSliderLow:SetText("");
	LFGMM_LfmTab_BroadcastMessagePreviewSliderHigh:SetText("");

	LFGMM_LfmTab_BroadcastMessageInfoButton:SetScript("OnClick", LFGMM_LfmTab_BroadcastMessageInfoButton_OnClick);

	LFGMM_LfmTab.SearchAnimationLock = false;

	LFGMM_LfmTab_UpdateBroadcastMessage();
end


function LFGMM_LfmTab_Show()
	PanelTemplates_SetTab(LFGMM_MainWindow, 2);

	LFGMM_LfgTab_BroadcastMessageInfoWindow:Hide();
	LFGMM_LfmTab_BroadcastMessageInfoWindow:Hide();
	LFGMM_SettingsTab_RequestInviteMessageInfoWindow:Hide();
	LFGMM_SettingsTab_ChannelsDropDownInfoWindow:Hide();
	LFGMM_ListTab_MessageInfoWindow_Hide();

	LFGMM_LfgTab:Hide();
	LFGMM_LfmTab:Show();
	LFGMM_ListTab:Hide();
	LFGMM_SettingsTab:Hide();

	LFGMM_MainWindowTab1:SetPoint ("CENTER", "LFGMM_MainWindow", "BOTTOMLEFT",    60, -14);
	LFGMM_MainWindowTab2:SetPoint ("CENTER", "LFGMM_MainWindow", "BOTTOMLEFT",   135, -17);
	LFGMM_MainWindowTab3:SetPoint ("CENTER", "LFGMM_MainWindow", "BOTTOMRIGHT", -140, -14);
	LFGMM_MainWindowTab4:SetPoint ("CENTER", "LFGMM_MainWindow", "BOTTOMRIGHT",  -60, -14);

	PlaySound(SOUNDKIT.IG_CHARACTER_INFO_TAB);

	LFGMM_LfmTab_Refresh();
	LFGMM_LfmTab_CategoryDropDown_Validate();
end


function LFGMM_LfmTab_Refresh()
	if (not LFGMM_LfmTab:IsVisible()) then
		return;
	end

	if (LFGMM_DB.SEARCH.LFM.Broadcast) then
		LFGMM_LfmTab_BroadcastMessageTemplateInputBox:Show();
		LFGMM_LfmTab_BroadcastMessageInfoButton:Show();
		LFGMM_LfmTab_BroadcastMessagePreview:Show();
		LFGMM_LfmTab_BroadcastMessagePreview_Refresh();
	else
		LFGMM_LfmTab_BroadcastMessageTemplateInputBox:Hide();
		LFGMM_LfmTab_BroadcastMessageInfoButton:Hide();
		LFGMM_LfmTab_BroadcastMessagePreview:Hide();
		LFGMM_LfmTab_BroadcastMessagePreviewSlider:Hide();
		LFGMM_LfmTab_BroadcastMessageInfoWindow:Hide();
	end

	local groupSize = table.getn(LFGMM_GLOBAL.GROUP_MEMBERS);
	local dungeonSize = (LFGMM_DB.SEARCH.LFM.Dungeon ~= nil and LFGMM_GLOBAL.DUNGEONS[LFGMM_DB.SEARCH.LFM.Dungeon].Size) or nil;
	if (dungeonSize and groupSize >= dungeonSize) then
		LFGMM_LfmTab_AutoStopCheckBox:SetChecked(false);
	else
		LFGMM_LfmTab_AutoStopCheckBox:SetChecked(LFGMM_DB.SEARCH.LFM.AutoStop);
	end

	if (LFGMM_DB.SEARCH.LFM.Running) then
		UIDropDownMenu_DisableDropDown(LFGMM_LfmTab_CategoryDropDown);
		UIDropDownMenu_DisableDropDown(LFGMM_LfmTab_DungeonsDropDown);
		UIDropDownMenu_DisableDropDown(LFGMM_LfmTab_ModeDropDown);
		LFGMM_LfmTab_SearchActiveText:Show();
		LFGMM_LfmTab_StartStopSearchButton:SetText("Stop searching");
		LFGMM_LfmTab_BroadcastMessageTemplateInputBox:Disable();
		LFGMM_Utility_ToggleEnabledColor(LFGMM_LfmTab_BroadcastMessageTemplateInputBox, false);
		LFGMM_Utility_ToggleEnabledColor(LFGMM_LfmTab_BroadcastMessagePreview, false);
		LFGMM_Utility_ToggleEnabledColor(LFGMM_LfmTab_MatchOnText, false);
		LFGMM_Utility_ToggleCheckBoxEnabled(LFGMM_LfmTab_MatchLfgCheckBox, false);
		LFGMM_Utility_ToggleCheckBoxEnabled(LFGMM_LfmTab_MatchLfmCheckBox, false);
		LFGMM_Utility_ToggleCheckBoxEnabled(LFGMM_LfmTab_MatchUnknownCheckBox, false);
		LFGMM_Utility_ToggleCheckBoxEnabled(LFGMM_LfmTab_AutoStopCheckBox, false);
		LFGMM_Utility_ToggleCheckBoxEnabled(LFGMM_LfmTab_IgnoreBoostsCheckBox, false);
		LFGMM_Utility_ToggleCheckBoxEnabled(LFGMM_LfmTab_EnableBroadcastCheckBox, false);
		LFGMM_LfmTab_StartAnimateSearchText();

	else
		UIDropDownMenu_EnableDropDown(LFGMM_LfmTab_CategoryDropDown);
		UIDropDownMenu_EnableDropDown(LFGMM_LfmTab_DungeonsDropDown);
		LFGMM_LfmTab_SearchActiveText:Hide();
		LFGMM_LfmTab_StartStopSearchButton:SetText("Start search");
		LFGMM_LfmTab_BroadcastMessageTemplateInputBox:Enable();
		LFGMM_Utility_ToggleEnabledColor(LFGMM_LfmTab_BroadcastMessageTemplateInputBox, true);
		LFGMM_Utility_ToggleEnabledColor(LFGMM_LfmTab_BroadcastMessagePreview, true);
		LFGMM_Utility_ToggleEnabledColor(LFGMM_LfmTab_MatchOnText, true);
		LFGMM_Utility_ToggleCheckBoxEnabled(LFGMM_LfmTab_MatchLfgCheckBox, true);
		LFGMM_Utility_ToggleCheckBoxEnabled(LFGMM_LfmTab_MatchLfmCheckBox, true);
		LFGMM_Utility_ToggleCheckBoxEnabled(LFGMM_LfmTab_MatchUnknownCheckBox, true);
		LFGMM_Utility_ToggleCheckBoxEnabled(LFGMM_LfmTab_EnableBroadcastCheckBox, true);
		LFGMM_Utility_ToggleCheckBoxEnabled(LFGMM_LfmTab_IgnoreBoostsCheckBox, true);

		local isHcAvailable = LFGMM_DB.SEARCH.LFM.Category == LFGMM_KEYS.DUNGEON_CATEGORIES.TBC;
		if isHcAvailable then
			UIDropDownMenu_EnableDropDown(LFGMM_LfmTab_ModeDropDown)
		else
			UIDropDownMenu_DisableDropDown(LFGMM_LfmTab_ModeDropDown);
		end

		if (dungeonSize and groupSize >= dungeonSize) then
			LFGMM_Utility_ToggleCheckBoxEnabled(LFGMM_LfmTab_AutoStopCheckBox, false);
		else
			LFGMM_Utility_ToggleCheckBoxEnabled(LFGMM_LfmTab_AutoStopCheckBox, true);
		end

		if (LFGMM_DB.SEARCH.LFM.Dungeon ~= nil) then
			LFGMM_LfmTab_StartStopSearchButton:Enable();
		else
			LFGMM_LfmTab_StartStopSearchButton:Disable();
		end
	end
end

function LFGMM_LfmTab_CategoryDropDown_OnInitialize(self)
	local createItem = function(categoryCode, categoryName)
		local item = UIDropDownMenu_CreateInfo();
		item.arg1 = categoryCode;
		item.text = categoryName;
		item.isNotRadio = true;
		item.checked = LFGMM_DB.SEARCH.LFM.Category == categoryCode
		item.keepShownOnClick = false;
		item.func = LFGMM_LfmTab_CategoryDropDown_Item_OnClick;
		return item
	end

	for _, category in ipairs(LFGMM_GLOBAL.CATEGORIES) do
		UIDropDownMenu_AddButton(createItem(category.Code, category.Name), 1);
	end

	LFGMM_LfmTab_CategoryDropDown_Validate();
	LFGMM_LfmTab_CategoryDropDown_UpdateText();
end

function LFGMM_LfmTab_ModeDropDown_OnInitialize(self)
	local createItem = function(mode, modeName)
		local item = UIDropDownMenu_CreateInfo();
		item.arg1 = mode;
		item.text = modeName;
		item.isNotRadio = true;
		item.checked = LFGMM_DB.SEARCH.LFM.Mode == mode;
		item.keepShownOnClick = false;
		item.func = LFGMM_LfmTab_ModeDropDown_Item_OnClick;
		return item
	end

	for _, mode in ipairs(LFGMM_GLOBAL.MODES) do
		UIDropDownMenu_AddButton(createItem(mode.Code, mode.Name), 1);
	end

	if LFGMM_DB.SEARCH.LFM.Mode == nil then
		LFGMM_DB.SEARCH.LFM.Mode = LFGMM_KEYS.DUNGEON_MODES.NONE;
	end

	LFGMM_LfmTab_ModeDropDown_UpdateText();
end

function LFGMM_LfmTab_DungeonsDropDown_OnInitialize(self, level)
	local dungeonsAndRaids = LFGMM_Utility_GetAvailableDungeonsAndRaidsMap();
	local dungeonMap = dungeonsAndRaids[LFGMM_DB.SEARCH.LFM.Category];
	LFGMM_LfmTab_DungeonsDropDown_Initialize_Internal(level, dungeonMap);
end

function LFGMM_LfmTab_DungeonsDropDown_Initialize_Internal(level, dungeonMap)
	local createSingleDungeonItem = function(dungeon)
		local item = LFGMM_Utility_CreateDungeonsDropdownItem(dungeon);
		item.keepShownOnClick = false;
		item.isNotRadio = true;
		item.checked = LFGMM_DB.SEARCH.LFM.Dungeon == dungeon.Index;
		item.func = LFGMM_LfmTab_DungeonsDropDown_Item_OnClick;
		UIDropDownMenu_AddButton(item, 1);
	end

	local createMultiDungeonItem = function(dungeon)
		-- Get available subdungeons
		local availableSubDungeons = {};
		for _,subDungeonIndex in ipairs(dungeon.SubDungeons) do
			if (LFGMM_Utility_IsDungeonAvailable(LFGMM_GLOBAL.DUNGEONS[subDungeonIndex])) then
				table.insert(availableSubDungeons, subDungeonIndex);
			end
		end

		local item = LFGMM_Utility_CreateDungeonsDropdownItem(dungeon);
		item.hasArrow = true;
		item.keepShownOnClick = false;
		item.isNotRadio = not LFGMM_Utility_ArrayContains(availableSubDungeons, LFGMM_DB.SEARCH.LFM.Dungeon)
		item.checked = LFGMM_DB.SEARCH.LFM.Dungeon == dungeon.Index or LFGMM_Utility_ArrayContains(availableSubDungeons, LFGMM_DB.SEARCH.LFM.Dungeon);
		item.value = { DungeonIndexes = availableSubDungeons, ParentDungeonIndex = dungeon.Index };
		item.func = LFGMM_LfmTab_DungeonsDropDown_Item_OnClick;
		UIDropDownMenu_AddButton(item, 1);
	end

	local createSubDungeonItem = function(dungeonIndex, parentDungeonIndex)
		local dungeon = LFGMM_GLOBAL.DUNGEONS[dungeonIndex];
		local item = LFGMM_Utility_CreateDungeonsDropdownItem(dungeon);
		item.keepShownOnClick = false;
		item.isNotRadio = LFGMM_DB.SEARCH.LFM.Dungeon ~= parentDungeonIndex;
		item.checked = LFGMM_DB.SEARCH.LFM.Dungeon == dungeon.Index or LFGMM_DB.SEARCH.LFM.Dungeon == parentDungeonIndex;
		item.func = LFGMM_LfmTab_DungeonsDropDown_Item_OnClick;
		UIDropDownMenu_AddButton(item, 2);
	end

	if dungeonMap == nil or #dungeonMap == 0 then
		-- No valid dungeons item
		local noDungeonsItem = UIDropDownMenu_CreateInfo();
		noDungeonsItem.text = "No available dungeons";
		noDungeonsItem.disabled = true;
		noDungeonsItem.notCheckable = true;
		UIDropDownMenu_AddButton(noDungeonsItem);
		return;
	end

	if level == 1 then
		for _, entry in ipairs(dungeonMap) do
			-- Dungeon headers
			if #dungeonMap > 1 and #entry.List > 0 then
				local dungeonsHeader = UIDropDownMenu_CreateInfo();
				dungeonsHeader.text = entry.Header;
				dungeonsHeader.isTitle = true;
				dungeonsHeader.notCheckable = true;
				UIDropDownMenu_AddButton(dungeonsHeader);
			end
	
			-- Dungeon menu items
			for _, dungeon in ipairs(entry.List) do
        if (dungeon.Enabled) then
          if (dungeon.ParentDungeon == nil) then
            if (dungeon.SubDungeons == nil) then
              createSingleDungeonItem(dungeon);
            else
              createMultiDungeonItem(dungeon);
            end
          end
        end
			end
		end
	elseif level == 2 then
		local entry = UIDROPDOWNMENU_MENU_VALUE;

		-- Sub dungeon menu items
		for _,dungeonIndex in ipairs(entry.DungeonIndexes) do
			createSubDungeonItem(dungeonIndex, entry.ParentDungeonIndex);
		end
	end

	-- Update search selection text
	LFGMM_LfmTab_DungeonsDropDown_UpdateText();
end

function LFGMM_LfmTab_CategoryDropDown_Item_OnClick(self, categoryCode)
	LFGMM_DB.SEARCH.LFM.Dungeon = nil;
	LFGMM_DB.SEARCH.LFM.Category = categoryCode;

	if LFGMM_DB.SEARCH.LFM.Category ~= LFGMM_KEYS.DUNGEON_CATEGORIES.TBC then
		LFGMM_DB.SEARCH.LFM.Mode = LFGMM_KEYS.DUNGEON_MODES.NONE;
	end

	LFGMM_LfmTab_CategoryDropDown_UpdateText();
	LFGMM_LfmTab_DungeonsDropDown_UpdateText();
	LFGMM_LfmTab_ModeDropDown_UpdateText();
	LFGMM_LfmTab_UpdateBroadcastMessage();
	LFGMM_LfmTab_Refresh();
	LFGMM_LfmTab_CategoryDropDown_Validate();
end

function LFGMM_LfmTab_CategoryDropDown_Validate()
	if (LFGMM_DB.SEARCH.LFM.Category == nil) then
		-- TODO: Clear dungeon selection
		UIDropDownMenu_DisableDropDown(LFGMM_LfmTab_DungeonsDropDown);
	end
end

function LFGMM_LfmTab_CategoryDropDown_UpdateText()
	if (LFGMM_DB.SEARCH.LFM.Category == nil) then
		UIDropDownMenu_SetText(LFGMM_LfmTab_CategoryDropDown, "<Select category>");
	else
		local category = LFGMM_Core_GetCategoryByCode(LFGMM_DB.SEARCH.LFM.Category);
		if category ~= nil then
			UIDropDownMenu_SetText(LFGMM_LfmTab_CategoryDropDown, category.Name);
		end
	end
end

function LFGMM_LfmTab_ModeDropDown_Item_OnClick(self, mode)
	LFGMM_DB.SEARCH.LFM.Mode = LFGMM_DB.SEARCH.LFM.Category == LFGMM_KEYS.DUNGEON_CATEGORIES.TBC and mode or nil
	LFGMM_LfmTab_ModeDropDown_UpdateText();
	LFGMM_LfmTab_UpdateBroadcastMessage();
	LFGMM_LfmTab_Refresh();
end

function LFGMM_LfmTab_ModeDropDown_UpdateText()
	local mode = LFGMM_Core_GetModeByCode(LFGMM_DB.SEARCH.LFM.Mode);
	UIDropDownMenu_SetText(LFGMM_LfmTab_ModeDropDown, mode.Name);
end

function LFGMM_LfmTab_DungeonsDropDown_Item_OnClick(self, dungeonIndex)
	-- Set dungeon
	LFGMM_DB.SEARCH.LFM.Dungeon = dungeonIndex;

	-- Update broadcast message
	LFGMM_LfmTab_UpdateBroadcastMessage();

	-- Update dungeon dropdown text
	LFGMM_LfmTab_DungeonsDropDown_UpdateText();

	-- Close drop down menus
	CloseDropDownMenus();

	-- Refresh
	LFGMM_LfmTab_Refresh();
end

function LFGMM_LfmTab_DungeonsDropDown_UpdateText()
	if (LFGMM_DB.SEARCH.LFM.Dungeon == nil) then
		UIDropDownMenu_SetText(LFGMM_LfmTab_DungeonsDropDown, "<Select dungeon>");
	else
		local dungeonName = LFGMM_GLOBAL.DUNGEONS[LFGMM_DB.SEARCH.LFM.Dungeon].Name;
		UIDropDownMenu_SetText(LFGMM_LfmTab_DungeonsDropDown, dungeonName);
	end
end


function LFGMM_LfmTab_StartStopSearchButton_OnClick()
	-- TODO: Find better solution
	-- Workaround to prevent false positives when using the HC filter.
	-- This will temporary disable the message buffer.
	if LFGMM_DB.SEARCH.LFM.Category == LFGMM_KEYS.DUNGEON_CATEGORIES.TBC and LFGMM_DB.SEARCH.LFM.Mode ~= LFGMM_KEYS.DUNGEON_MODES.NONE then
		LFGMM_GLOBAL.MESSAGES = {};
	end

	PlaySound(SOUNDKIT.GS_LOGIN);

	-- Close drop down
	CloseDropDownMenus();

	if (LFGMM_DB.SEARCH.LFM.Running) then
		LFGMM_DB.SEARCH.LFM.Running = false;
		LFGMM_PopupWindow_Hide();
		LFGMM_BroadcastWindow_CancelBroadcast();
		LFGMM_MainWindowTab1:Show();
		LFGMM_Core_RemoveUnavailableDungeonsFromSelections();

	else
		-- Reset ignored messages
		for _,message in pairs(LFGMM_GLOBAL.MESSAGES) do
			message.Ignore = {};
		end

		-- Determine if autostop is available or not
		local groupSize = table.getn(LFGMM_GLOBAL.GROUP_MEMBERS);
		local dungeonSize = LFGMM_GLOBAL.DUNGEONS[LFGMM_DB.SEARCH.LFM.Dungeon].Size;
		if (groupSize >= dungeonSize) then
			LFGMM_GLOBAL.AUTOSTOP_AVAILABLE = false;
		else
			LFGMM_GLOBAL.AUTOSTOP_AVAILABLE = true;
		end

		-- Start search
		LFGMM_DB.SEARCH.LFM.Running = true;
		LFGMM_MainWindowTab1:Hide();

		-- Start broadcast
		if (LFGMM_DB.SEARCH.LFM.Broadcast) then
			LFGMM_BroadcastWindow_StartBroadcast();
		end

		-- Search for group match after 2 seconds
		C_Timer.After(2, LFGMM_Core_FindSearchMatch);
	end

	-- Refresh
	LFGMM_LfmTab_Refresh();
	LFGMM_MinimapButton_Refresh();
end


function LFGMM_LfmTab_MatchLfgCheckBox_OnClick()
	LFGMM_LfmTab_MatchLfgCheckBox:SetChecked(true);
end


function LFGMM_LfmTab_MatchLfmCheckBox_OnClick()
	LFGMM_DB.SEARCH.LFM.MatchLfm = LFGMM_LfmTab_MatchLfmCheckBox:GetChecked();
end


function LFGMM_LfmTab_MatchUnknownCheckBox_OnClick()
	LFGMM_DB.SEARCH.LFM.MatchUnknown = LFGMM_LfmTab_MatchUnknownCheckBox:GetChecked();
end


function LFGMM_LfmTab_AutoStopCheckBox_OnClick()
	LFGMM_DB.SEARCH.LFM.AutoStop = LFGMM_LfmTab_AutoStopCheckBox:GetChecked();
end

function LFGMM_LfmTab_IgnoreBoostsCheckBox_OnClick()
	LFGMM_DB.SEARCH.LFM.IgnoreBoosts = LFGMM_LfmTab_IgnoreBoostsCheckBox:GetChecked();
end


function LFGMM_LfmTab_EnableBroadcastCheckBox_OnClick()
	LFGMM_DB.SEARCH.LFM.Broadcast = LFGMM_LfmTab_EnableBroadcastCheckBox:GetChecked();

	-- Refresh
	LFGMM_LfmTab_Refresh();
end


function LFGMM_LfmTab_UpdateBroadcastMessage()
	local message = LFGMM_LfmTab_BroadcastMessageTemplateInputBox:GetText();

	-- Store template
	LFGMM_DB.SEARCH.LFM.BroadcastMessageTemplate = message;

	local placeHolder = "<dungeon>";
	local dungeonText = placeHolder;
	local abbreviationText = placeHolder;
	local lookingForNumberText = "<number>";

	-- Get dungeon
	if (LFGMM_DB.SEARCH.LFM.Dungeon ~= nil) then
		local dungeon = LFGMM_GLOBAL.DUNGEONS[LFGMM_DB.SEARCH.LFM.Dungeon];
		dungeonText = dungeon.Name;

		abbreviationText = dungeon.Abbreviation;

		-- Get number of players
		local lookingForNumber = dungeon.Size - table.getn(LFGMM_GLOBAL.GROUP_MEMBERS);
		if (lookingForNumber < 0) then
			lookingForNumber = 0;
		end
		lookingForNumberText = tostring(lookingForNumber);
	end

	-- Generate message
	message = string.gsub(message, "{[Dd]}", dungeonText);
	message = string.gsub(message, "{[Aa]}", abbreviationText);
	message = string.gsub(message, "{[Nn]}", lookingForNumberText);
	message = string.sub(message, 1, 255);

	if message ~= nil and message ~= placeHolder and LFGMM_DB.SEARCH.LFM.Mode == LFGMM_KEYS.DUNGEON_MODES.HC or  LFGMM_DB.SEARCH.LFM.Mode == LFGMM_KEYS.DUNGEON_MODES.NHC then
		message = message .. " (" .. LFGMM_DB.SEARCH.LFM.Mode .. ")";
	end

	-- Store message
	LFGMM_DB.SEARCH.LFM.BroadcastMessage = message;

	-- Update preview
	LFGMM_LfmTab_BroadcastMessagePreview_Refresh();
end


function LFGMM_LfmTab_BroadcastMessagePreview_Refresh()
	if (not LFGMM_LfmTab_BroadcastMessagePreview:IsVisible()) then
		return;
	end

	local text = LFGMM_DB.SEARCH.LFM.BroadcastMessage;
	local textLength = string.len(text);
	local maxLength = 40;

	if (textLength > maxLength) then
		LFGMM_LfmTab_BroadcastMessagePreviewSlider:SetMinMaxValues(1, textLength - maxLength + 1);
		local sliderPosition = LFGMM_LfmTab_BroadcastMessagePreviewSlider:GetValue();
		text = string.sub(text, sliderPosition, sliderPosition + maxLength - 1);
		LFGMM_LfmTab_BroadcastMessagePreviewSlider:Show();
	else
		LFGMM_LfmTab_BroadcastMessagePreviewSlider:SetValue(1);
		LFGMM_LfmTab_BroadcastMessagePreviewSlider:Hide();
	end

	LFGMM_LfmTab_BroadcastMessagePreview:SetText(text);
end


function LFGMM_LfmTab_StartAnimateSearchText()
	-- Ensure lock
	if (not LFGMM_LfgTab.SearchAnimationLock) then
		LFGMM_LfgTab.SearchAnimationLock = true;

		-- Start animation
		LFGMM_LfmTab_AnimateSearchingText();
	end
end


function LFGMM_LfmTab_AnimateSearchingText()
	-- Release lock if hidden or search is stopped
	if (not LFGMM_DB.SEARCH.LFM.Running or not LFGMM_LfmTab_SearchActiveText:IsVisible() or not LFGMM_MainWindow:IsVisible()) then
		LFGMM_LfgTab.SearchAnimationLock = false;
		return;
	end

	-- Get next string value
	LFGMM_LfmTab_SearchActiveText.StringAnimation = LFGMM_LfmTab_SearchActiveText.StringAnimation .. ".";
	if (string.len(LFGMM_LfmTab_SearchActiveText.StringAnimation) > 3) then
		LFGMM_LfmTab_SearchActiveText.StringAnimation = "";
	end

	-- Update text
	LFGMM_LfmTab_SearchActiveText:SetText("Searching" .. LFGMM_LfmTab_SearchActiveText.StringAnimation);

	-- Queue next update
	C_Timer.After(0.5, LFGMM_LfmTab_AnimateSearchingText)
end


function LFGMM_LfmTab_BroadcastMessageInfoButton_OnClick()
	if (LFGMM_LfmTab_BroadcastMessageInfoWindow:IsVisible()) then
		LFGMM_LfmTab_BroadcastMessageInfoWindow:Hide();
	else
		LFGMM_LfgTab_BroadcastMessageInfoWindow:Hide();
		LFGMM_LfmTab_BroadcastMessageInfoWindow:Show();
		LFGMM_SettingsTab_RequestInviteMessageInfoWindow:Hide();
		LFGMM_SettingsTab_ChannelsDropDownInfoWindow:Hide();
		LFGMM_ListTab_MessageInfoWindow:Hide();
	end
end

